/*****************************************************************************
 * Copyright 2008 Dustin Long (dlong@progmatism.com)
 *
 * This file is part of Paragrasp.
 *
 * Paragrasp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Paragrasp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Paragrasp.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/
var ParagraspConfig =
{
    startConfig : function()
    {
	this.loadRadioForeground();
	this.loadRadioBackground();

	this.loadCurrentKeyBindings();

	this.loadScrollSettings();

	var backColorGroup = document.getElementById("backColorGroup");
	backColorGroup.addEventListener('command', function (e) { ParagraspConfig.onBackColorGroup_Select(e); }, true);

	var customBackColor = document.getElementById("customBackColor");
	customBackColor.setAttribute("onchange", "ParagraspConfig.onCustomBackColor_Change();");

	var foreColorGroup = document.getElementById("foreColorGroup");
	foreColorGroup.addEventListener('command', function (e) { ParagraspConfig.onForeColorGroup_Select(e); }, true);

	var customForeColor = document.getElementById("customForeColor");
	customForeColor.setAttribute("onchange", "ParagraspConfig.onCustomForeColor_Change();");
    },

    loadCurrentKeyBindings : function()
    {
	this.setKeyBindButton("paragrasp.key.navigateUp", "keyUpButton");
	this.setKeyBindButton("paragrasp.key.navigateDown", "keyDownButton");
	this.setKeyBindButton("paragrasp.key.startHighlight", "keyStartButton");
	this.setKeyBindButton("paragrasp.key.clearHighlight", "keyClearButton");
	this.setKeyBindButton("paragrasp.key.automaticNav", "keyAutoButton");
    },

    loadScrollSettings : function()
    {
	var method = ParagraspShared.getStringPref("paragrasp.scroll.method");

	// stupid backwards compatability hack!
	// viva le Raymond Chen!
	if (ParagraspShared.getStringPref("paragrasp.advopt.scrollTop") == "1")
	{
            method = "1";
	}

	var scrollMethodGroup = document.getElementById("scrollMethodGroup");
	if (method == "2")
	{
            scrollMethodGroup.selectedIndex = 2;
	}
	else if (method == "1")
	{
            scrollMethodGroup.selectedIndex = 1;
	}
	else /* must (method == "0") */
	{
            scrollMethodGroup.selectedIndex = 0;
	}
	
	var wheel = ParagraspShared.getStringPref("paragrasp.scroll.wheel");
	var chkMouseWheel = document.getElementById("chkMouseWheel");
	if (wheel == "1")
	{
            chkMouseWheel.checked = true;
	}
	var smooth = ParagraspShared.getStringPref("paragrasp.scroll.smooth");
	var chkSmoothScroll = document.getElementById("chkSmoothScroll");
	if (smooth == "1")
	{
            chkSmoothScroll.checked = true;
	}
    },


    saveScrollSettings : function()
    {
	var scrollMethodGroup = document.getElementById("scrollMethodGroup");

	var method = new String(scrollMethodGroup.selectedIndex);
	
	ParagraspShared.setStringPref("paragrasp.scroll.method", method);
	
	ParagraspShared.setStringPref("paragrasp.advopt.scrollTop", "0");

	var chkMouseWheel = document.getElementById("chkMouseWheel");
	var wheel = (chkMouseWheel.checked ? "1" : "0");
	ParagraspShared.setStringPref("paragrasp.scroll.wheel", wheel);

	var chkSmoothScroll = document.getElementById("chkSmoothScroll");
	var smooth = (chkSmoothScroll.checked ? "1" : "0");
	ParagraspShared.setStringPref("paragrasp.scroll.smooth", smooth);
    },


    readableFromKeyValue : function(keyValue)
    {
	var build = "";

	var settings = keyValue.split("][");
	
	if (settings[0].indexOf("control") != -1)
            build += "Ctrl+";
	if (settings[0].indexOf("shift") != -1)
            build += "Shift+";
	if (settings[0].indexOf("alt") != -1)
            build += "Alt+";
	if (settings[1])
            build += settings[1];
	if (settings[2])
            build += ParagraspShared.nameOfKeyCode(settings[2]);
	
	return build;
    },

    setKeyBindButton : function(prefName, buttonId)
    {
	var keyValue = ParagraspShared.getStringPref(prefName);
	var readable = this.readableFromKeyValue(keyValue);
	var buttonNode = document.getElementById(buttonId);    
	buttonNode.label = readable;
    },

    loadRadioForeground : function()
    {
	var color = ParagraspShared.getStringPref("paragrasp.color.foreground");
	var foreColorGroup = document.getElementById("foreColorGroup");
	var customForeColor = document.getElementById("customForeColor");

	if (color == "#000000") {
            foreColorGroup.selectedIndex = 0;
	} else if (color == "#c00000") {
            foreColorGroup.selectedIndex = 1;
	} else if (color == "#0000c0") {
            foreColorGroup.selectedIndex = 2;
	} else if (color == "#00c000") {
            foreColorGroup.selectedIndex = 3;
	} else {
            foreColorGroup.selectedIndex = 4;
            customForeColor.disabled = false;
            customForeColor.value = color;
	}
    },

    loadRadioBackground : function()
    {
	var color = ParagraspShared.getStringPref("paragrasp.color.background");
	var backColorGroup = document.getElementById("backColorGroup");
	var customBackColor = document.getElementById("customBackColor");

	if (color == "#ffff88") {
            backColorGroup.selectedIndex = 0;
	} else if (color == "#88ff88") {
            backColorGroup.selectedIndex = 1;
	} else if (color == "#ff88ff") {
            backColorGroup.selectedIndex = 2;
	} else if (color == "#88ffff") {
            backColorGroup.selectedIndex = 3;
	} else {
            backColorGroup.selectedIndex = 4;
            customBackColor.disabled = false;
            customBackColor.value = color;
	}
    },

    saveRadioForeground : function()
    {
	var color = ParagraspShared.getStringPref("paragrasp.color.foreground");
	var foreColorGroup = document.getElementById("foreColorGroup");
	var customForeColor = document.getElementById("customForeColor");

	if ((foreColorGroup.selectedIndex == 0) && (color != "#000000")) {
            ParagraspShared.setStringPref("paragrasp.color.foreground", "#000000");
	} else if ((foreColorGroup.selectedIndex == 1) && (color != "#c00000")) {
            ParagraspShared.setStringPref("paragrasp.color.foreground", "#c00000");
	} else if ((foreColorGroup.selectedIndex == 2) && (color != "#0000c0")) {
            ParagraspShared.setStringPref("paragrasp.color.foreground", "#0000c0");
	} else if ((foreColorGroup.selectedIndex == 3) && (color != "#00c000")) {
            ParagraspShared.setStringPref("paragrasp.color.foreground", "#00c000");
	} else if ((foreColorGroup.selectedIndex == 4) && (color != customForeColor.value)) {
            ParagraspShared.setStringPref("paragrasp.color.foreground", customForeColor.value);
	}
    },

    saveRadioBackground : function()
    {
	var color = ParagraspShared.getStringPref("paragrasp.color.background");
	var backColorGroup = document.getElementById("backColorGroup");
	var customBackColor = document.getElementById("customBackColor");

	if ((backColorGroup.selectedIndex == 0) && (color != "#ffff88")) {
            ParagraspShared.setStringPref("paragrasp.color.background", "#ffff88");
	} else if ((backColorGroup.selectedIndex == 1) && (color != "#88ff88")) {
            ParagraspShared.setStringPref("paragrasp.color.background", "#88ff88");
	} else if ((backColorGroup.selectedIndex == 2) && (color != "#ff88ff")) {
            ParagraspShared.setStringPref("paragrasp.color.background", "#ff88ff");
	} else if ((backColorGroup.selectedIndex == 3) && (color != "#88ffff")) {
            ParagraspShared.setStringPref("paragrasp.color.background", "#88ffff");
	} else if ((backColorGroup.selectedIndex == 4) && (color != customBackColor.value)) {
            ParagraspShared.setStringPref("paragrasp.color.background", customBackColor.value);
	}
    },

    onBackColorGroup_Select : function(event)
    {
	var selected = event.target.id;
	var customBackColor = document.getElementById("customBackColor");

	if (selected == "optBackCustom")
	{
            customBackColor.disabled = false;
            if (customBackColor.value == "")
            {
		customBackColor.value = "#000000";
            }
	}
	else
	{
            customBackColor.disabled = true;
	}
    },

    onForeColorGroup_Select : function(event)
    {
	var selected = event.target.id;
	var customForeColor = document.getElementById("customForeColor");

	if (selected == "optForeCustom")
	{
            customForeColor.disabled = false;
            if (customForeColor.value == "")
            {
		customForeColor.value = "#ffffff";
            }
	}
	else
	{
            customForeColor.disabled = true;
	}
    },

    onCustomBackColor_Change : function()
    {
	var customBackColor = document.getElementById("customBackColor");

	color = this.formProperColor(customBackColor.value);

	if (color == null)
	{
            color = "#000000";
	}

	customBackColor.value = color;
    },

    onCustomForeColor_Change : function()
    {
	var customForeColor = document.getElementById("customForeColor");

	color = this.formProperColor(customForeColor.value);

	if (color == null)
	{
            color = "#ffffff";
	}

	customForeColor.value = color;
    },

    formProperColor : function(color)
    {
	var offset = 0;
	if (color[offset] == "#")
	{
            offset++;
	}

	var build = "#";

	var k;
	for (k = 0; k < 6; k++)
	{
            var ch = color[offset + k];
            if (((ch >= '0') && (ch <= '9')) || ((ch >= 'a') && (ch <= 'f')))
            {
		build = build + color[offset + k];
            } else if ((ch >= 'A') && (ch <= 'F')) {
		build = build + color[offset + k].toLowerCase();
            } else {
		break;
            }
	}
	
	if (k != 6)
	{
            return null;
	}
	
	return build;
    },

    advancedConfig : function(name)
    {
	var result = new Object();
	result.value = null;
	window.openDialog('chrome://paragrasp/content/advconf.xul', '_blank', 'chrome,dialog=yes,modal=yes,centerscreen=yes', name, result);
    },

    keySetConfig : function(name)
    {
	var result = new Object();
	result.value = null;
	window.openDialog('chrome://paragrasp/content/keyset.xul', '_blank', 'chrome,dialog=yes,modal=yes,centerscreen=yes', name, result);
	this.onKeyChange(name, result.value);
    },

    onKeyChange : function(name, newValue)
    {
	var keyValue = ParagraspShared.fetchKeyValue(name);

	// did the key actually get changed?
	if ((newValue != null) && (keyValue != newValue))
	{
            keyValue = newValue;
	    
	    var configMessage;
	    
            // error message
            configMessage = document.getElementById("configMessage");
            configMessage.value = "An error occured while trying to change the key shortcut";

            // find the applicable button
            var buttonId = ParagraspShared.fetchButtonId(name);

            // change the button's label
            var readable = this.readableFromKeyValue(keyValue);
            var buttonNode = document.getElementById(buttonId);
            buttonNode.label = readable;

            // change the preference
            var prefName = ParagraspShared.fetchPrefName(name);
            ParagraspShared.setStringPref(prefName, keyValue);

            // warning message
            configMessage = document.getElementById("configMessage");
            configMessage.value = "Key changes will only affect new windows";
	}
    },

    finishConfig : function()
    {
	this.saveRadioForeground();
	this.saveRadioBackground();

	this.saveScrollSettings();

	window.close();
    },

    dismissConfig : function()
    {
	window.close();
    }
};

