/*****************************************************************************
 * Copyright 2008 Dustin Long (dlong@progmatism.com)
 *
 * This file is part of Paragrasp.
 *
 * Paragrasp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Paragrasp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Paragrasp.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/
var ParagraspEntry = {
    OnLoad: function() {
        // Initialization code.
        addEventListener("keypress", function(e) {
            return ParagraspEntry.OnKeyPress(e);
        }, true);
        this.RebindKeyShortcuts();
        this.PopulateContextMenu();
    },

    DoBindKey: function(keyValue, keyId) {
        var settings = keyValue.split("][");

        var keyNode = document.getElementById(keyId);

        if (settings[0])
            keyNode.setAttribute("modifiers", settings[0]);
        if (settings[1])
            keyNode.setAttribute("key", settings[1]);
        if (settings[2])
            keyNode.setAttribute("keycode", settings[2]);
    },

    // When called within onLoad, will successfully change key shortcuts.
    RebindKeyShortcuts : function() {
        this.SetKeyPref("navigateUp", "key_paragraspNavigateUp");
        this.SetKeyPref("navigateDown", "key_paragraspNavigateDown");
        this.SetKeyPref("clearHighlight", "key_paragraspClearHighlight");
        this.SetKeyPref("startHighlight", "key_paragraspStartHighlight");
        this.SetKeyPref("automaticNav", "key_paragraspAutomaticNav");
    },

    SetKeyPref : function(pref, key) {
        var setting = ParagraspShared.getStringPref("paragrasp.key." + pref);
        this.DoBindKey(setting, key);
    },

    g_popupNode : null,

    OnPopupCreate : function(event) {
        this.g_popupNode = this.g_currMouseNode;
    },

    PopulateContextMenu : function() {
        var contextMenu = document.getElementById("contentAreaContextMenu");
        contextMenu.addEventListener("popupshowing", function(e) {
            return ParagraspEntry.OnPopupCreate(e)
        }, false);
    },

    g_currMouseNode : null,

    OnMouseScroll : function(event) {
        ParagraspUtil.RetrieveAdvOpt();
        if (ParagraspUtil.MouseWheelPref() == "1") {
            var currWin = ParagraspUtil.GetCurrentTab();
            var m_hlNodeRange = ParagraspUtil.GetPrivate("hlNodeRange");

            if (m_hlNodeRange != undefined) {
                if (event.detail > 0) {
                    this.OnKeyNavi(event, 1);
                } else if (event.detail < 0) {
                    this.OnKeyNavi(event, -1);
                } else {
                    return;
                }
                event.preventDefault();
            }
        }
    },

    OnMouseOver : function(event) {
        var node = ((event.target) ? event.target : event.srcElement);
        if ((node) && (this.g_currMouseNode != node)) {
            this.g_currMouseNode = node;
        }
    },

    OnMouseDown : function(event) {
        // Check for [Ctrl+Middle_Click].
        if (event.button != 1) return;

        if (event.ctrlKey == false) return;

        var node = this.g_currMouseNode;

        ParagraspCore.JumpToNode(node);

        event.preventDefault();
        event.stopPropagation();
    },

    OnKeyAuto: function(event) {
        var auto = this.getStatusAutoNavi();
        auto = !auto;
        this.setStatusAutoNavi(auto);

        var menu_auto = document.getElementById("menu_auto");
        menu_auto.setAttribute("checked", auto);
    },

    OnHere : function(event) {
        var node = this.g_popupNode;
        ParagraspCore.JumpToNode(node);
    },

    OnKeyNavi: function(event, direction) {
        // TODO: Fix me!!!
        //try {
            ParagraspUtil.RetrieveAdvOpt();
            ParagraspCore.NavigateParagraph(direction);
            var container = ParagraspUtil.GetCurrentTab().document.
                documentElement;
            this.RemoveMouseUntilItMoves(container);
        //} catch (e) {
        //    alert(e);
        //}
    },

    OnKeyClear: function(event, direction) {
        // Get per-tab data.
        var m_oldColors = ParagraspUtil.GetPrivate("oldColors");
        var m_hlNodeRange = ParagraspUtil.GetPrivate("hlNodeRange");

        // Get rid of old highlight.
        ParagraspCore.EraseCurrentHighlight(m_hlNodeRange, m_oldColors);

        ParagraspUtil.SetPrivate("oldColors", undefined);
        ParagraspUtil.SetPrivate("hlNodeRange", undefined);
    },

    OnKeyStart: function(event) {
        var node = this.g_currMouseNode;

        ParagraspCore.JumpToNode(node);
    },

    // Should probably go in util.
    g_statusAutoNavi : null,

    getStatusAutoNavi : function() {
        if (this.g_statusAutoNavi == null) {
            this.g_statusAutoNavi = false;
        }
        return this.g_statusAutoNavi;
    },

    setStatusAutoNavi : function(value) {
        this.g_statusAutoNavi = value;
    },


    OnKeyPress: function(event) {
        ParagraspUtil.RetrieveAdvOpt();

        if (this.getStatusAutoNavi()) {
            if ((event.altKey == false) && (event.ctrlKey == false) &&
                (event.metaKey == false) && (event.shiftKey == false)) {
                if (event.keyCode == 40) {
                    // Down.
                    this.NavigateParagraph(1);
                    event.stopPropagation();
                    event.preventDefault();
                } else if (event.keyCode == 38) {
                    // Up.
                    this.NavigateParagraph(-1);
                    event.stopPropagation();
                    event.preventDefault();
                }
            }
        }
    },

    OnKeyJump: function(event) {
        ParagraspUtil.RetrieveAdvOpt();

        // Get per-tab data.
        var currWin = ParagraspUtil.GetCurrentTab();
        var m_oldColors = ParagraspUtil.GetPrivate("oldColors");
        var m_hlNodeRange = ParagraspUtil.GetPrivate("hlNodeRange");

        var targetRange = null;

        var select = currWin.getSelection();

        var middle = select.anchorNode;


        var values;
        var edge;

        // This is ugly. A better idea would be refactor this function, put
        // most of it into ParagraspCore, and combine it with JumpToNode.
        var that = ParagraspCore;

        var point = that.MoveBackward(that.MoveForward(middle));

        for (;;) {
            targetRange = that.AttemptBorderParagraph(point, function(node) {
                return that.MoveBackward(node);
            });
            if (targetRange != null) {
                targetRange = that.TravelForwardParagraph(
                    that.lastElement(targetRange));
                break;
            }
            targetRange = that.AttemptBorderParagraph(point, function(node) {
                return that.MoveForward(node);
            });
            if (targetRange != null) {
                targetRange = that.TravelBackwardParagraph(
                    that.firstElement(targetRange));
                break;
            }
            break;
        }

        // If there are nodes in that direction.
        if (targetRange != null) {
            // Get rid of old highlight.
            that.EraseCurrentHighlight(m_hlNodeRange, m_oldColors);

            // Make text fields into real nodes.
            m_hlNodeRange = that.PrepareDomForMod(targetRange);

            // Return the old colors for later.
            m_oldColors = that.HighlightNodes(m_hlNodeRange);

            that.ScrollWindowTo(m_hlNodeRange, null, false);
        }

        // Get rid of old highlight.
        ParagraspUtil.SetPrivate("oldColors", m_oldColors);
        ParagraspUtil.SetPrivate("hlNodeRange", m_hlNodeRange);
    },

    CursorData : function() {
        return "AAACAAEAEBAAAAAAAABoBAAAFgAAACgAAAAQAAAAIAAAAAEAIAAAAAAAQA" +
        "QAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAAA" +
        "AAAAAAAAAAAAAAAAAAAAAAAAA//8AAP//AAD//wAA//8AAP//AAD//wAA//8AAP//" +
        "AAD//wAA//8AAP//AAD//wAA//8AAP//AAD//wAA//8AAA==";
    },

    m_mouseListenState : 0,

    // Would a better solution be to only add the mousemove listener once?
    // Do I want it to stick around forever?
    // It seems like this is better, assuming someone paragrasps for a while,
    // then uses the mouse for a while.
    RemoveMouseUntilItMoves : function(container) {
        if (container.style) {
            if (this.m_mouseListenState == 0) {
                container.style.cursor = "url(data:image/gif;base64," +
                this.CursorData() + "), pointer";
                window.addEventListener(
                    "mousemove",
                    ParagraspEntry.RestoreMouseAfterItMoves(container, window),
                    false);
                this.m_mouseListenState = 1;
            }
        }
    },

    RestoreMouseAfterItMoves : function(container, host) {
        var that = this;
        var response = function(e) {
            if (container.style.cursor != null) {
                container.style.cursor = null;
            }
            that.m_mouseListenState = 0;
            host.removeEventListener("mousemove", response, false);
        }
        return response;
    },
};

window.addEventListener('DOMMouseScroll', function(e) {
    return ParagraspEntry.OnMouseScroll(e)
} , false);
window.addEventListener("load", function(e) {
    return ParagraspEntry.OnLoad(window, e);
}, false);
window.addEventListener('mousedown', function(e) {
    return ParagraspEntry.OnMouseDown(e);
}, true);
document.addEventListener("mouseover", function(e) {
    return ParagraspEntry.OnMouseOver(e);
}, false);
