/*****************************************************************************
 * Copyright 2008 Dustin Long (dlong@progmatism.com)
 *
 * This file is part of Paragrasp.
 *
 * Paragrasp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Paragrasp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Paragrasp.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/
var ParagraspCore = {
    min : function(a, b) {
        if (a < b) return a;
        return b;
    },

    max : function(a, b) {
        if (a > b) return a;
        return b;
    },

    firstElement : function(ls) {
        return ls[0];
    },

    lastElement : function(ls) {
        return ls[ls.length - 1];
    },

    GetNodeCoordinates : function(/*@NotNull*/ node) {
        var target = node;

        if (node.nodeType == document.TEXT_NODE) {
            return null;
        }

        if ((!target.offsetWidth) || (!target.offsetHeight)) {
            return null;
        }

        var x0 = target.offsetLeft;
        var y0 = target.offsetTop;
        var iter = target;
        while (iter.offsetParent &&
               (iter.offsetParent != document.body)) {
            iter = iter.offsetParent;
            x0 += iter.offsetLeft;
            y0 += iter.offsetTop;
        }
        var x1 = x0 + target.offsetWidth;
        var y1 = y0 + target.offsetHeight;

        // Work around for bug 338384.
        // Node only returns one line's worth of height when calculating
        // offsetHeight. Instead I find the next node and subtract the
        // difference of y positions.
        if ((target.parentNode.clientHeight) &&
            (target.parentNode.clientHeight == 0)) {
            var nextNode = target.nextSibling;
            if ((nextNode != null) && (nextNode.offsetTop)) {
                y1 = y0 + (nextNode.offsetTop - target.offsetTop);
            }
        }

        var make = new Array();

        make.push(y0);
        make.push(x1);
        make.push(y1);
        make.push(x0);

        return make;
    },

    GetRangeCoordinates : function(/*@NotNull*/ range) {
        var first = range[0];
        var last = range[range.length - 1];

        var firstcoor, lastcoor;
        firstcoor = this.GetNodeCoordinates(first);
        lastcoor = (first == last) ? firstcoor : this.GetNodeCoordinates(last);

        if (firstcoor == null || lastcoor == null) { return null; }

        var y0 = this.min(firstcoor[0], lastcoor[0]);
        var x1 = this.max(firstcoor[1], lastcoor[1]);
        var y1 = this.max(firstcoor[2], lastcoor[2]);
        var x0 = this.min(firstcoor[3], lastcoor[3]);

        var make = new Array();

        make.push(y0);
        make.push(x1);
        make.push(y1);
        make.push(x0);

        return make;
    },

    GetRangeTop : function(/*@NotNull*/ range) {
        var firstcoor = this.GetNodeCoordinates(range[0]);
        if (firstcoor) {
            return firstcoor[0];
        }
        else {
            return null;
        }
    },

    highlightColor : function() {
        return ParagraspShared.getStringPref("paragrasp.color.background");
    },

    foregroundColor : function() {
        return ParagraspShared.getStringPref("paragrasp.color.foreground");
    },

    NotableElement : function(/*@NotNull*/ node) {
        if (this.m_traveling) {
            if (this.NodeNameIsIn(node, "|h1|h2|h3|h4|h5|h6|h7|h8|h9|")) {
                return true;
            }
            if ((ParagraspUtil.ListElemPref() != "1") &&
                (this.NodeNameIsIn(node, "|ul|dl|ol|"))) {
                return true;
            } else if ((ParagraspUtil.ListElemPref() == "1") &&
                       (this.NodeNameIsIn(node, "|li|dd|"))) {
                return true;
            }
        }

        return false;
    },

    ParagraphIsInnermost : function(/*@NotNull*/ node) {
        if (this.NotableElement(node)) {
            return true;
        }

        if (node.nodeName.toLowerCase() != "p") {
            return false;
        }

        return this.HasSolidContents(node);
    },

    HasSolidContents : function(node) {
        var children = node.childNodes;

        var count = 0;

        var k = 0;
        for (;;) {
            var child = children.item(k);
            if (child == null) {
                return true;
            }
            if ((child.nodeName != null) &&
                (child.nodeName.toLowerCase() == "p")) {
                return false;
            }
            if ((child.nodeName != null) &&
                (child.nodeName.toLowerCase() == "br")) {
                count++;
                if (count >= 2) {
                    return false;
                }
            }
            if ((child.nodeName != null) &&
                (child.nodeName.toLowerCase() == "font")) {
                if (!this.HasSolidContents(child)) {
                    return false;
                }
            }
            k++;
        }
    },

    MIN_TEXT_SIZE : 16,

    IsJustTextChild : function(/*@NotNull*/ node) {
        if (node.nodeType == document.TEXT_NODE) {
            return true;
        } else {
            return false;
        }
    },

    IsLoneTextChild : function(/*@NotNull*/ node) {
        if ((node.nodeType == document.TEXT_NODE) &&
            (node.previousSibling == null) &&
            (node.nextSibling == null)) {
            return true;
        } else {
            return false;
        }
    },

    // http://www.htmlhelp.com/reference/wilbur/font/sub.html
    FORMAT_NODES : "|b|i|strong|em|font|a|span|sub|sup|strike|u|" +
                   "tt|big|small|dfn|kbd|var|",

    IsSimpleFormatParent : function(/*@NotNull*/ node) {
        if ((node.nodeType == document.ELEMENT_NODE) &&
            (node.previousSibling == null) &&
            (node.nextSibling == null) &&
            (node.firstChild != null)) {
            return (this.NodeNameIsIn(node, this.FORMAT_NODES));
        } else {
            return false;
        }
    },

    IsJustFormatNode : function(/*@NotNull*/ node) {
        if (node.nodeType == document.ELEMENT_NODE) {
            return (this.NodeNameIsIn(node, this.FORMAT_NODES));
        } else {
            return false;
        }
    },

    IsHeavyFormatNode : function(/*@NotNull*/ node) {
        if (this.IsJustFormatNode(node)) {
            var iter = node.firstChild;
            if (iter == null) {
                return false;
            }
            for (;;) {
                if (iter == null) {
                    return true;
                } else if (this.IsJustTextChild(iter)) {
                    /*pass*/
                } else if (this.IsMoveOverNode(iter)) {
                    /*pass*/
                } else if (!this.IsHeavyFormatNode(iter)) {
                    return false;
                }
                iter = iter.nextSibling;
            }
        }
        return false;
    },

    Unreadable : function(/*@NotNull*/ html) {
        // Some people, when confronted with a problem, think "I know,
        // I'll use regular expressions." Now they have two problems. -jwz
        return (html.match(/^(\s|&nbsp;|\<br\/?\>)*$/));
    },

    IsNonBlankText : function(/*@NotNull*/ node) {
        if ((node.nodeType == document.TEXT_NODE) &&
            (!this.Unreadable(node.nodeValue))) {
            return true;
        } else {
            return false;
        }
    },

    IsBlankText : function(/*@NotNull*/ node) {
        if ((node.nodeType == document.TEXT_NODE) &&
            (this.Unreadable(node.nodeValue))) {
            return true;
        } else {
            return false;
        }
    },

    StepForward : function(node) {
        if (node.childNodes.length != 0) {
            return node.firstChild;
        } else if (node.nextSibling != null) {
            return node.nextSibling;
        } else {
            var iter = node;

            for (;;) {
                if (iter == null) {
                    return null;
                }
                if (iter.nextSibling != null) {
                    break;
                }
                iter = iter.parentNode;
            }
            return iter.nextSibling;
        }
    },

    StepBackward : function(node) {
        if (node.previousSibling != null) {
            var iter = node.previousSibling;
            while (iter.childNodes.length != 0) {
                iter = iter.lastChild;
            }
            return iter;

        } else if (node.parentNode != null) {
            return node.parentNode;
        } else {
            return null;
        }
    },

    WalkForward : function(node) {
        var iter = node;
        if ((iter != null) &&
            (this.ParagraphIsInnermost(iter))) {
            while (iter.lastChild != null) {
                iter = iter.lastChild;
            }
        }
        iter = this.StepForward(iter);
        if (iter != null) {
            if (this.IsLoneTextChild(iter) || this.IsSimpleFormatParent(iter)) {
                while ((iter != null) && (this.IsSimpleFormatParent(iter))) {
                    iter = this.StepForward(iter);
                }
                iter = this.StepForward(iter);
            }
        }
        return iter;
    },

    WalkBackward : function(node) {
        var iter = node;
        iter = this.StepBackward(iter);
        var parent = iter;
        for (;;) {
            if (parent == null) {
                break;
            } else if (this.ParagraphIsInnermost(parent)) {
                iter = parent;
                break;
            }
            parent = parent.parentNode;
        }
        if (iter != null) {
            if (this.IsLoneTextChild(iter)) {
                iter = this.StepBackward(iter);
                while ((iter != null) && (this.IsSimpleFormatParent(iter))) {
                    iter = this.StepBackward(iter);
                }
            }
        }
        return iter;
    },

    IsMoveOverNode : function(/*@NotNull*/ node) {
        return ((node.nodeName.toLowerCase() == "img") ||
                (node.nodeName.toLowerCase() == "head") ||
                (node.nodeType == node.COMMENT_NODE));
    },

    MoveForward : function(node) {
        var iter = node;
        for (;;) {
            iter = this.WalkForward(iter);
            if (iter == null) {
                break;
            }
            if (!this.IsMoveOverNode(iter)) {
                break;
            }
        }
        return iter;
    },

    MoveBackward : function(node) {
        var iter = node;
        for (;;) {
            iter = this.WalkBackward(iter);
            break;
        }
        return iter;
    },

    NODE_TYPE_NULL : 0,
    NODE_TYPE_PARA : 1,
    NODE_TYPE_TEXT : 2,
    NODE_TYPE_BREK : 3,
    NODE_TYPE_EMPT : 4,
    NODE_TYPE_INVS : 5,

    ClassifyNode : function(node) {
        if (node == null) {
            return this.NODE_TYPE_NULL;
        } else if (this.ParagraphIsInnermost(node)) {
            return this.NODE_TYPE_PARA;
        } else if (this.IsNonBlankText(node)) {
            return this.NODE_TYPE_TEXT;
        } else if ((node.firstChild != null) &&
                   (this.IsNonBlankText(node.firstChild)) &&
                   (this.IsLoneTextChild(node.firstChild))) {
            return this.NODE_TYPE_TEXT;
        } else if (this.IsHeavyFormatNode(node)) {
            return this.NODE_TYPE_TEXT;
        } else if (node.nodeName.toLowerCase() == "br") {
            return this.NODE_TYPE_BREK;
        } else if (this.IsBlankText(node)) {
            return this.NODE_TYPE_EMPT;
        } else {
            return this.NODE_TYPE_INVS;
        }
    },

    // Before and including version 1.04 this function did not exist.
    // I'm not sure it does what I need it to do.
    LeaveUnit : function(node, move) {
        var iter = node;

        var kind = this.ClassifyNode(iter);

        var ahead;

        var buttress = false;

        if (kind == this.NODE_TYPE_PARA) {
            buttress = true;
        } else if (kind == this.NODE_TYPE_TEXT) {
            ahead = this.TravelUnit(iter, move);
            kind = this.ClassifyNode(ahead);
            if ((kind == this.NODE_TYPE_BREK) ||
                (kind == this.NODE_TYPE_PARA)) {
                buttress = true;
            }
        }

        iter = this.TravelUnit(iter, move);

        var kind = this.ClassifyNode(iter);

        while ((kind == this.NODE_TYPE_INVS) || (kind == this.NODE_TYPE_BREK)) {
            iter = this.TravelUnit(iter, move);
            kind = this.ClassifyNode(iter);
        }

        return new Array(buttress, iter);
    },

    LeaveSeries : function(node, move) {
        var iter = node;

        var kind = this.ClassifyNode(iter);

        var ahead;

        var buttress = false;

        if (kind == this.NODE_TYPE_PARA) {
            buttress = true;
        } else if (kind == this.NODE_TYPE_TEXT) {
            ahead = this.TravelUnit(iter, move);
            kind = this.ClassifyNode(ahead);
            if ((kind == this.NODE_TYPE_BREK) ||
                (kind == this.NODE_TYPE_PARA)) {
                buttress = true;
            }
        }

        iter = this.TravelSeries(iter, move);

        var kind = this.ClassifyNode(iter);

        while ((kind == this.NODE_TYPE_INVS) || (kind == this.NODE_TYPE_BREK)) {
            iter = this.TravelSeries(iter, move);
            kind = this.ClassifyNode(iter);
        }

        return new Array(buttress, iter);
    },

    // Move once, skipping empty.
    TravelUnit : function(node, move) {
        var point = node;
        var kind;

        for (;;) {
            point = move(point);

            kind = this.ClassifyNode(point);

            if (kind != this.NODE_TYPE_EMPT) {
                break;
            }
        }

        return point;
    },

    // Move once, then check the kind.
    // Move until the last node of that same kind (except for br, p).
    TravelSeries : function(node, move) {
        if (node == null) {
            return null;
        }

        var iter = this.TravelUnit(node, move);

        if (iter == null) {
            return null;
        }

        var kind = this.ClassifyNode(iter);

        if ((kind == this.NODE_TYPE_PARA) || (kind == this.NODE_TYPE_BREK)) {
            return iter;
        }

        var next, prev;

        for (;;) {
            prev = iter;

            iter = this.TravelUnit(iter, move);

            next = this.ClassifyNode(iter);

            if (next != kind) {
                return prev;
            }
        }

        return null;
    },

    m_traveling : false,

    TravelToParagraph : function(node, move, garbage) {
        var point = node;

        var range;

        var values;

        var from;
        var start;

        var edge;
        var buttress;

        var normalNavigation = true;

        for (;;) {
            for (;;) {
                // True unless we just left a false edge.
                if (normalNavigation) {
                    // Just passed a true edge, remember where you came from.
                    start = point;

                    // Move once, then move until we are not in a blank series.
                    values = this.LeaveSeries(point, move);
                    buttress = values[0];
                    point = values[1];
                } else {
                    normalNavigation = true;
                }

                if (point == null) {
                    return null;
                }

                edge = this.FindIfAtEdge(point, move, buttress);

                if (edge == null) {
                    // Fall off the end of the world.
                    return null;
                } else if (edge == 0) {
                    // Success!
                    break;
                } else if (edge != -1) {
                    // A false edge, just keep on moving.
                    point = edge;
                    normalNavigation = false;
                }

                this.m_traveling = false;
            }

            range = this.BuildRangeBetween(start, point, move);

            if (range == null) {
                return null;
            }

            if (!this.VisibleRange(range)) {
                continue;
            }

            break;
        }

        return range;
    },

    FindIfAtEdge : function(point, move, buttress) {
        var kind_curr, kind_next, kind_ahead;
        var next, ahead;

        kind_curr = this.ClassifyNode(point);

        next = this.TravelUnit(point, move);
        kind_next = this.ClassifyNode(next);

        if (kind_curr == this.NODE_TYPE_TEXT) {
            // Text node is only ok if:
            //   1) Followed by a para or at least two brek's.
            //   2) Followed by a invs/null and preceeded by a buttress.
            if (kind_next == this.NODE_TYPE_PARA) {
                return 0;
            } else if (kind_next == this.NODE_TYPE_BREK) {
                ahead = this.TravelUnit(next, move);
                kind_ahead = this.ClassifyNode(ahead);
                if ((kind_ahead == this.NODE_TYPE_BREK) ||
                    (kind_ahead == this.NODE_TYPE_PARA)) {
                    return 0;
                } else if (kind_ahead == this.NODE_TYPE_TEXT) {
                    // Containing only a single break is a false edge.
                    return this.TravelSeries(next, move);
                } else if (kind_ahead == this.NODE_TYPE_INVS) {
                    return 0;
                }
            } else if ((kind_next == this.NODE_TYPE_INVS) ||
                       (kind_next == this.NODE_TYPE_NULL)) {
                if (buttress) {
                    return 0;
                }
            } else {
                // In the middle of a text block, keep going.
                return this.TravelSeries(next, move);
            }
        } else if (kind_curr == this.NODE_TYPE_PARA) {
            if ((kind_next == this.NODE_TYPE_INVS) ||
                (kind_next == this.NODE_TYPE_NULL)) {
                if (buttress) {
                    return 0;
                }
            } else {
                return 0;
            }
        } else {
            return null;
        }

        return -1;
    },

    TravelForwardParagraph : function(lsNodes) {
        var self = this;
        var move = function(node) {
            return ParagraspCore.MoveForward(node);
        };
        return this.TravelToParagraph(lsNodes, move, true);
    },

    TravelBackwardParagraph : function(lsNodes) {
        var self = this;
        var move = function(node) {
            return ParagraspCore.MoveBackward(node);
        };
        return this.TravelToParagraph(lsNodes, move, false);
    },

    AttemptBorderParagraph : function(point, move) {
        var borderRange;

        var values;
        var edge;

        for (;;) {
            if (point == null) {
                return null;
            }

            edge = this.FindIfAtEdge(point, move, false);
            if (edge == 0) {
                break;
            } else if (edge == -1) {
                return null;
            } else if (edge != null) {
                point = edge;
            } else {
                // Before and including version 1.04 this branch was never
                // reached.
                values = this.LeaveUnit(point, move);
                point = values[1];
            }
        }

        borderRange = this.TravelToParagraph(point, move, false);
        return borderRange;
    },

    VisibleRange : function(/*@NotNull*/ range) {
        var coords = this.GetRangeCoordinates(range);

        if (coords != null) {
            // We can't figure out the coordinates, assume yes.
            var top = coords[0];
            var right = coords[1];
            var bottom = coords[2];
            var left = coords[3];

            if ((right <= 0) ||        // Right side off viewport.
                (bottom <= 0) ||       // Bottom side off viewport.
                (right - left < 10) || // Very thin.
                (bottom - top < 10)) { // Very short.
                return false;
            }
        }

        if (ParagraspUtil.IncShortiesPref() != "1") {
            var count = 0;

            var inc;

            for (var k = 0; k < range.length; k++) {
                var elem = range[k];
                if (elem.textContent) {
                    inc = elem.textContent.length;
                    count += inc;
                } else if (elem.innerHtml) {
                    inc = elem.innerHtml.length;
                    count += inc;
                }
            }

            if (count < this.MIN_TEXT_SIZE) {
                return false;
            }
        }

        return true;
    },

    // Return the range of all visible nodes in the "paragraph".
    // For a p tag, this is only the p node itself.
    // Otherwise, it's a list of nodes, not including the trailing br tags.
    BuildRangeBetween : function(start, end, move) {
        var make = new Array();

        var iter = this.TravelUnit(start, move);

        var ahead;

        var kind;

        for (;;) {
            kind = this.ClassifyNode(iter);
            if ((kind == this.NODE_TYPE_TEXT) ||
                (kind == this.NODE_TYPE_PARA)) {
                make.push(iter);
            }

            if (iter == end) {
                break;
            }

            iter = this.TravelUnit(iter, move);
        }

        if (move == this.MoveBackward) {
            make.reverse();
        }

        return make;
    },

    EraseCurrentHighlight : function(lsNodes, lsOld) {
        if (lsOld == null) {
            return;
        }

        var k;

        for (k = 0; k < lsNodes.length; k++) {
            lsNodes[k].style.backgroundColor = lsOld[k * 2];
            lsNodes[k].style.color = lsOld[k * 2 + 1];
        }
    },

    WrapNodeInto : function(nakedNode, replaceNode, targetNode, accumList) {
        var clone = nakedNode.cloneNode(true);

        var wrap;

        if (ParagraspUtil.NoSpansPref() != "1") {
            wrap = ParagraspUtil.CreateElement("span");
        } else {
            // Horrible awful hack available to fix certain webpages' broken
            // css. Seriously, why would you ever make a blanket rule for
            // span tags?!?!
            wrap = ParagraspUtil.CreateElement("div");
        }
        wrap.appendChild(clone);

        targetNode.replaceChild(wrap, replaceNode);
        accumList.push(wrap);
        return wrap;
    },

    SplitEols : function(text, frontOut, restOut, linesOut) {
        var index = text.indexOf("\n\n");

        if (index != -1) {
            var count = 2;
            for (var k = index + 2; k < text.length; k++) {
                if (text[k] != '\n') { break; }
                count++;
            }

            var front = ParagraspUtil.CreateTextNode(text.substr(0, index));
            var rest  = ParagraspUtil.CreateTextNode(
                text.substr(index + count, text.length - index - count));
            frontOut.value = front;
            restOut.value  = rest;
            linesOut.value = count;

            return true;
        }
        return false;
    },

    // Thought i needed a wrapper to handle cases where 'where==null',
    // but apparently the native function handles it correctly.
    InsertNodeBefore : function(/*@NotNull*/ parent, /*@NotNull*/ node,
                                /*?Nullable*/ where) {
        parent.insertBefore(node, where);
    },

    PrepareDomForMod : function(lsNodes) {
        var make = new Array();

        var parent;

        var wrap;

        var node;

        var style;

        var clone;

        var k;

        for (k = 0; k < lsNodes.length; k++) {
            node = lsNodes[k];

            parent = node.parentNode;

            if (node.nodeType == document.TEXT_NODE) {
                if (this.NodeNameIsIn(node.parentNode, "|pre|")) {
                    var content = node.textContent;

                    var frontObj = new Object();
                    var restObj = new Object();
                    var linesObj = new Object();
                    if (this.SplitEols(content, frontObj, restObj, linesObj)) {
                        var into = this.WrapNodeInto(frontObj.value,
                                                     node, parent, make);
                        var space = into.nextSibling;

                        for (var k = 0; k < linesObj.value; k++) {
                            var br = ParagraspUtil.CreateElement("br");
                            this.InsertNodeBefore(parent, br, space);
                        }
                        this.InsertNodeBefore(parent, restObj.value, space);
                    } else {
                        this.WrapNodeInto(node, node, parent, make);
                    }
                    // For cases where the pre node contains formatting
                    // break as soon as we finish one split, since the split
                    // will invalidate our target node.
                    break;
                } else {
                    this.WrapNodeInto(node, node, parent, make);
                }
            } else if (this.NodeNameIsIn(node, "|pre|")) {
                var content = node.textContent;

                var frontObj = new Object();
                var restObj = new Object();
                var linesObj = new Object();
                if (this.SplitEols(content, frontObj, restObj, linesObj)) {
                    var into = this.WrapNodeInto(frontObj.value,
                                                 node.firstChild, node, make);
                    var space = into.nextSibling;

                    for (var k = 0; k < linesObj.value; k++) {
                        var br = ParagraspUtil.CreateElement("br");
                        this.InsertNodeBefore(node, br, space);
                    }
                    this.InsertNodeBefore(node, restObj.value, space);
                } else {
                    make.push(node);
                }
            } else {
                make.push(node);
            }
        }

        return make;
    },

    NodeNameIsIn : function(/*?Nullable*/ node, namesList) {
        if ((node) && (node.nodeName)) {
            var search = "|" + node.nodeName.toLowerCase() + "|";
            return (namesList.indexOf(search) != -1);
        }
        return false;
    },

    HighlightNodes : function(lsNodes) {
        var make = new Array();

        var k;

        for (k = 0; k < lsNodes.length; ++k) {

            var node = lsNodes[k];

            make.push(node.style.backgroundColor);
            make.push(node.style.color);

            node.style.backgroundColor = this.highlightColor();
            node.style.color = this.foregroundColor();
        }

        return make;
    },

    // To help smooth scrolling keep up.
    g_fakeWinTop : null,

    ScrollWindowTo : function(lsNodes, atTop, modded) {
        if (lsNodes == null) {
            return;
        }

        var scrollNode = lsNodes[0];

        var coords = this.GetRangeCoordinates(lsNodes);

        if (coords == null) {
            return;
        }

        var currWin = ParagraspUtil.WindowObject();

        var winTop = currWin.pageYOffset;
        if (this.g_fakeWinTop) {
            winTop = this.g_fakeWinTop;
        }
        var winLeft = currWin.pageXOffset;
        var winHeight = currWin.innerHeight;
        var winBottom = winTop + winHeight;

        var targetTop = coords[0];
        var targetHeight = coords[2] - coords[0];

        var centerOffset = targetTop + (targetHeight - winHeight) / 2;

        if (centerOffset < 0) { centerOffset = 0; }

        var scrollPref =
            ParagraspShared.getStringPref("paragrasp.scroll.method");
        var smoothPref =
            ParagraspShared.getStringPref("paragrasp.scroll.smooth");

        if (scrollPref == "2") { // Scroll to same height.
            if (atTop == null) {
                return;
            } else if ((atTop < winTop) || (atTop > winBottom)) {
                this.DestinationScroll(currWin, winLeft, winTop,
                                       targetTop - 10, smoothPref == "1");
            } else {
                this.DestinationScroll(currWin, winLeft, winTop,
                                       winTop + (targetTop - atTop),
                                       smoothPref == "1");
            }
        } else if (scrollPref == "1") { // Scroll to top.
            this.DestinationScroll(currWin, winLeft, winTop,
                                   targetTop - 10, smoothPref == "1");
        } else if (scrollPref == "0") { // Scroll to center.
            this.DestinationScroll(currWin, winLeft, winTop,
                                   centerOffset, smoothPref == "1");
        }
    },

    DestinationScroll : function(theWindow, winLeft, winTop, destTop, smooth) {
        if (smooth) {
            var steps = 5;
            this.g_fakeWinTop = destTop;
            this.StepBasedScroll(theWindow, winLeft, winTop,
                                 destTop, steps - 1, steps);
        } else {
            theWindow.scrollTo(winLeft, destTop);
        }
    },

    StepBasedScroll : function(theWindow, winLeft, winTop,
                               destTop, count, start) {
        if (count < 0) {
            this.g_fakeWinTop = null;
            return;
        } else {
            var self = this;
            theWindow.scrollTo(winLeft,
                               (winTop - destTop) * (count / start) + destTop);
            setTimeout(function() {
                self.StepBasedScroll(theWindow, winLeft, winTop,
                                     destTop, count - 1, start);
            }, 0);
        }
    },

    DefaultCurrentNode : function() {
        return ParagraspUtil.DocumentRootNode();
    },

    DefaultNodeRange : function() {
        var tailNode = this.DefaultCurrentNode();
        var range = new Array();
        range.push(tailNode);
        return range;
    },

    NavigateParagraph : function(direction) {
        var m_oldColors = ParagraspUtil.GetPrivate("oldColors");
        var m_hlNodeRange = ParagraspUtil.GetPrivate("hlNodeRange");

        try {
            this.m_traveling = true;

            if (m_hlNodeRange == undefined) {
                m_hlNodeRange = this.DefaultNodeRange();
                this.m_traveling = false;
            }

            // Pick the direction to move in.
            var targetRange;
            if (direction == -1) {
                targetRange = this.TravelBackwardParagraph(
                    this.firstElement(m_hlNodeRange));
            } else { // must (direction == 1)
                targetRange = this.TravelForwardParagraph(
                    this.lastElement(m_hlNodeRange));
            }

            // If there are nodes in that direction.
            if (targetRange != null) {
                var top = this.GetRangeTop(m_hlNodeRange);

                // Get rid of old highlight.
                this.EraseCurrentHighlight(m_hlNodeRange, m_oldColors);

                // Make text fields into real nodes.
                m_hlNodeRange = this.PrepareDomForMod(targetRange);

                // Return the old colors for later.
                m_oldColors = this.HighlightNodes(m_hlNodeRange);

                this.ScrollWindowTo(m_hlNodeRange, top, false);
            }
        } catch (e) {
            alert("Paragrasp Error: " + e + " on line " + e.lineNumber);
        }

        ParagraspUtil.SetPrivate("oldColors", m_oldColors);
        ParagraspUtil.SetPrivate("hlNodeRange", m_hlNodeRange);
    },

    JumpToNode : function(node) {
        if (node == null) { return; }

        // Get private data.
        var m_oldColors = ParagraspUtil.GetPrivate("oldColors");
        var m_hlNodeRange = ParagraspUtil.GetPrivate("hlNodeRange");

        var targetRange = null;

        var values;
        var edge;

        var point = this.MoveBackward(this.MoveForward(node));
        var self = this;
        for (;;) {
            targetRange = this.AttemptBorderParagraph(point, function (node) {
                return self.MoveBackward(node);
            });
            if (targetRange != null) {
                targetRange = this.TravelForwardParagraph(
                    this.lastElement(targetRange));
                break;
            }
            targetRange = this.AttemptBorderParagraph(point, function (node) {
                return self.MoveForward(node);
            });
            if (targetRange != null) {
                targetRange = this.TravelBackwardParagraph(
                    this.firstElement(targetRange));
                break;
            }
            break;
        }

        if (targetRange == null) {
            if ((point.nodeName) && (point.nodeName.toLowerCase() == "pre")) {
                targetRange = new Array();
                targetRange.push(point);
            }
        }

        if (targetRange == null) {
            alert("Could not find a paragraph");
            return;
        }

        // If there are nodes in that direction.
        if (targetRange != null) {
            // Get rid of old highlight.
            this.EraseCurrentHighlight(m_hlNodeRange, m_oldColors);

            // Make text fields into real nodes.
            m_hlNodeRange = this.PrepareDomForMod(targetRange);

            // Return the old colors for later.
            m_oldColors = this.HighlightNodes(m_hlNodeRange);

            this.ScrollWindowTo(m_hlNodeRange, null, false);
        }

        // Get rid of old highlight.
        ParagraspUtil.SetPrivate("oldColors", m_oldColors);
        ParagraspUtil.SetPrivate("hlNodeRange", m_hlNodeRange);
    },

    ClearHighlight : function() {
        // Get private data.
        var m_oldColors = ParagraspUtil.GetPrivate("oldColors");
        var m_hlNodeRange = ParagraspUtil.GetPrivate("hlNodeRange");

        // Get rid of old highlight.
        this.EraseCurrentHighlight(m_hlNodeRange, m_oldColors);

        // Get rid of old highlight.
        ParagraspUtil.SetPrivate("oldColors", undefined);
        ParagraspUtil.SetPrivate("hlNodeRange", undefined);
    },
};
