/*****************************************************************************
 * Copyright 2008 Dustin Long (dlong@progmatism.com)
 *
 * This file is part of Paragrasp.
 *
 * Paragrasp is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.

 * Paragrasp is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.

 * You should have received a copy of the GNU General Public License
 * along with Paragrasp.  If not, see <http://www.gnu.org/licenses/>.
 *****************************************************************************/
var ParagraspShared =
{
    gKeyPrefs : null,
    gKeyIds : null,
    gKeyNames : null,
    gScanCodes : null,

    init : function()
    {
	this.gKeyPrefs = new Array();
	this.gKeyPrefs[0] = "paragrasp.key.navigateUp";
	this.gKeyPrefs[1] = "paragrasp.key.navigateDown";
	this.gKeyPrefs[2] = "paragrasp.key.clearHighlight";
	this.gKeyPrefs[3] = "paragrasp.key.startHighlight";
	this.gKeyPrefs[4] = "paragrasp.key.automaticNav";
	this.gKeyPrefs[5] = "paragrasp.key.jumpToSelect";

	this.gKeyIds = new Array();
	this.gKeyIds[0] = "key_paragraspNavigateUp";
	this.gKeyIds[1] = "key_paragraspNavigateDown";
	this.gKeyIds[2] = "key_paragraspClearHighlight";
	this.gKeyIds[3] = "key_paragraspStartHighlight";
	this.gKeyIds[4] = "key_paragraspAutomaticNav";
	this.gKeyIds[5] = "key_paragraspJumpToSelect";

	this.gKeyNames = new Array();
	this.gKeyNames[0] = "up";
	this.gKeyNames[1] = "down";
	this.gKeyNames[2] = "clear";
	this.gKeyNames[3] = "start";
	this.gKeyNames[4] = "auto";
	this.gKeyNames[5] = "jump";

	this.gKeyButtonIds = new Array();
	this.gKeyButtonIds[0] = "keyUpButton";
	this.gKeyButtonIds[1] = "keyDownButton";
	this.gKeyButtonIds[2] = "keyClearButton";
	this.gKeyButtonIds[3] = "keyStartButton";
	this.gKeyButtonIds[4] = "keyAutoButton";
	this.gKeyButtonIds[5] = "keyJumpButton";

	this.gScanCodes = new Array();
	this.gScanCodes[0] = "VK_UP";
	this.gScanCodes[1] = "VK_RIGHT";
	this.gScanCodes[2] = "VK_DOWN";
	this.gScanCodes[3] = "VK_LEFT";
	this.gScanCodes[4] = "VK_PAGE_UP";
	this.gScanCodes[5] = "VK_PAGE_DOWN";
	this.gScanCodes[6] = "VK_HOME";
	this.gScanCodes[7] = "VK_END";
	this.gScanCodes[8] = "VK_TAB";
	this.gScanCodes[9] = "VK_ESCAPE";
    },

    fetchKeyValue : function(name)
    {
	var k = 0;
	for (k = 0; k < 5; k++)
	{
            if (this.gKeyNames[k] == name)
            {
		var prefs = this.gKeyPrefs[k];
		return this.getStringPref(prefs);
            }
	}
	return "";
    },

    fetchButtonId : function(name)
    {
	var k = 0;
	for (k = 0; k < 5; k++)
	{
            if (this.gKeyNames[k] == name)
            {
		var id = this.gKeyButtonIds[k];
		return id;
            }
	}
	return "";
    },

    fetchPrefName : function(name)
    {
	var k = 0;
	for (k = 0; k < 5; k++)
	{
            if (this.gKeyNames[k] == name)
            {
		var prefName = this.gKeyPrefs[k];
		return prefName;
            }
	}
	return "";
    },

    nameOfKeyCode : function(code)
    {
	if (code == "VK_UP")
            return "up";
	if (code == "VK_RIGHT")
            return "right";
	if (code == "VK_DOWN")
            return "down";
	if (code == "VK_LEFT")
            return "left";
	if (code == "VK_PAGE_UP")
            return "page up";
	if (code == "VK_PAGE_DOWN")
            return "page down";
	if (code == "VK_HOME")
            return "home";
	if (code == "VK_END")
            return "end";
	if (code == "VK_TAB")
            return "tab";
	if (code == "VK_ESCAPE")
            return "escape";
	return "[" + code + "]";
    },

    dropListIndexOfScanCode : function(code)
    {
	var k;
	for (k = 0; k < 10; k++)
	{
            if (this.gScanCodes[k] == code)
            {
		return k;
            }
	}
	return -1;
    },

    scanCodeOfDropListIndex : function(index)
    {
	return this.gScanCodes[index];
    },

    nameOfDropListIndex : function(index)
    {
	return this.nameOfKeyCode(this.gScanCodes[index]);
    },

    m_Prefs : null,

    getStringPref : function(aPrefstring) 
    {
	if (this.m_Prefs == null)
	{
            this.m_Prefs = Components.classes['@mozilla.org/preferences;1']
	                             .getService(Components.interfaces.nsIPrefBranch);
	}
	try {
            return this.m_Prefs.getCharPref(aPrefstring);
	}
	catch(e) {
            alert("ERROR: " + e);
	}
	return null;
    },

    setStringPref : function(aPrefstring, aValue) 
    {
	if (this.m_Prefs == null)
	{
            this.m_Prefs = Components.classes['@mozilla.org/preferences;1']
	                             .getService(Components.interfaces.nsIPrefBranch);
	}
	try {
            this.m_Prefs.setCharPref(aPrefstring, aValue);
	}
	catch(e) {
            alert("ERROR: " + e);
	}
	return null;
    }

};
ParagraspShared.init();
