# -*- mode:python -*-
#
# Copyright 2008 Dustin Long (dlong@progmatism.com)
#
# This file is part of Paragrasp.
#
# Paragrasp is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
#
# Paragrasp is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with Paragrasp.  If not, see <http://www.gnu.org/licenses/>.
#
#
# This is a bpriori intermediary file. Run "bpr -i paragrasp.bpr.i" to generate
# a build file.

[##############################################################################
 # Copy javascript source for Firefox.
 ##############################################################################
 {'type': 'rule_barrier', 'rules': ['jar', 'amo', 'unpacked', 'jar-signed']},

 # Copy javascript source.
 {'type': 'copy_action', 'out': 'jar/', 'in': ['js/'], 'access': 'contents'},

 ##############################################################################
 # Create jar and manifest meta file for Firefox.
 ##############################################################################
 {'type': 'rule_barrier', 'rules': ['jar', 'amo', 'jar-signed']},

 # Create jar
 {'type': 'zip_action', 'out': 'chrome/paragrasp.jar', 'in': ['jar/'],
  'access': 'contents', 'param': ['-r', '-0']},

 # Create meta files for jar / amo.
 {'type': 'custom_action', 'out': 'chrome.manifest',
  'in': ['paragrasp.conf', 'firefox-template.manifest'],
  'src': 'actions/create_manifest_action.py', 'param': ['-p', 'jar']},

 ##############################################################################
 # Create install rdf meta file for Firefox.
 ##############################################################################
 {'type': 'rule_barrier', 'rules': ['amo']},

 # Create meta files for amo only
 {'type': 'custom_action', 'out': 'install.rdf',
  'in': ['paragrasp.conf', 'firefox-template.rdf'],
  'src': 'actions/create_manifest_action.py', 'param': ['-p', 'no_update']},

 ##############################################################################
 # Copy unpacked directory and manifest meta file for Firefox.
 ##############################################################################
 {'type': 'rule_barrier', 'rules': ['unpacked']},

 # Copy js into paragrasp directory.
 {'type': 'copy_action', 'out': 'chrome/paragrasp/', 'in': ['jar/'],
  'access': 'contents'},

 # Create meta files for unpacked.
 {'type': 'custom_action', 'out': 'chrome.manifest',
  'in': ['paragrasp.conf', 'firefox-template.manifest'],
  'src': 'actions/create_manifest_action.py', 'param': ['-p', 'unpacked']},

 ##############################################################################
 # Create install rdf meta file for Firefox.
 ##############################################################################
 {'type': 'rule_barrier', 'rules': ['jar', 'unpacked', 'jar-signed']},

 # Create meta files for jar / unpacked
 {'type': 'custom_action', 'out': 'install.rdf',
  'in': ['paragrasp.conf', 'firefox-template.rdf'],
  'src': 'actions/create_manifest_action.py', 'param': ['-p', 'jar']},

 ##############################################################################
 # Create xpi.
 ##############################################################################
 {'type': 'rule_barrier', 'rules': ['jar', 'amo', 'unpacked', 'jar-signed']},

 # Create the xpi.
 {'type': 'copy_action', 'out': 'defaults/preferences/paragrasp.js',
  'in': ['preferences/paragrasp.js']},
 {'type': 'zip_action', 'out': 'paragrasp.xpi',
  'in': ['chrome/', 'chrome.manifest', 'defaults/', 'install.rdf'],
  'param': ['-r', '-9']},

 # Deploy the paragrasp extension for jar, amo, unpacked.
 {'type': 'rule_barrier', 'rules': ['jar', 'amo', 'unpacked']},

 {'type': 'build_output', 'in': ['paragrasp.xpi']},

 ##############################################################################
 # Sign rdf for secure updates.
 ##############################################################################
 {'type': 'rule_barrier', 'rules': ['jar-signed']},

 {'type': 'custom_action', 'out': 'update-unsigned.rdf',
  'in': ['paragrasp.conf', 'firefox-update-template.php', 'paragrasp.xpi'],
  'src': 'actions/create_manifest_action.py', 'param': ['-p', 'hash']},

 {'type': 'custom_action', 'out': 'update.php',
  'in': ['update-unsigned.rdf', 'private/firefox.key.db'],
  'src': 'actions/sign_with_mccoy.py'},

 {'type': 'build_output', 'in': ['update.php', 'paragrasp.xpi']},

 ##############################################################################
 # Build extension for Chromium.
 ##############################################################################
 {'type': 'rule_barrier', 'rules': ['chrome-signed']},

 {'type': 'copy_action', 'out': 'chromium/paragrasp.js',
  'in': ['js/content/paragrasp.js']},

 # Copy chromium source.
 {'type': 'copy_action', 'out': 'chromium/', 'in': ['chromium-src/'],
  'access': 'contents'},

 # Create chrome manifest.
 {'type': 'custom_action', 'out': 'chromium/manifest.json',
  'in': ['paragrasp.conf', 'chromium-template.json'],
  'src': 'actions/create_manifest_action.py'},

 # Sign the chrome extension.
 {'type': 'custom_action', 'out': 'paragrasp.crx',
  'in': ['chromium/', 'private/chromium.key.pem'],
  'src': 'actions/package_chrome_extension.sh'},

 {'type': 'custom_action', 'out': 'updates.xml',
  'in': ['paragrasp.conf', 'chromium-update-template.xml'],
  'src': 'actions/create_manifest_action.py'},

 {'type': 'build_output', 'in': ['updates.xml', 'paragrasp.crx']},
]
